import os
from pathlib import Path
from transformers import AutoModelForCausalLM
import torch
from huggingface_hub import HfApi

def upload_to_hub(directory, root_path):
    # Configure these variables
    username = "Anonymous"  # Replace with your Hugging Face username
    repo_name = "attacks" + str(directory)[len(root_path):].replace('/', '-')  # Naming the repo after the directory
    
    try:
        hf_api = HfApi()
        model_info = hf_api.model_info(username + '/' + repo_name)
        print("Model ID:", model_info.modelId)
        return
    except Exception as e:
        print(e)
        model = AutoModelForCausalLM.from_pretrained(directory, torch_dtype=torch.bfloat16, device_map='cpu')
        private = True  # Set to True to create a private repository
        print(repo_name)
        # Create a repository on Hugging Face Hub
        # if repo exists, skip
        model.push_to_hub(repo_name, organization=None, private=private)

def find_and_upload_subfolders_with_safetensors(root_path):
    # Walk through the directory tree starting from root_path
    for root, dirs, files in os.walk(root_path):
        if any(file.endswith('.safetensors') for file in files):
            upload_to_hub(Path(root), root_path)

# Example usage
if __name__ == "__main__":
    root_path = "output"  # Replace with the path to the folder to be scanned
    find_and_upload_subfolders_with_safetensors(root_path)